@@ -20,7 +20,7 @@ import logging
 from typing import List, Optional
 from urllib.parse import urlparse
 
-from pydantic import BaseSettings, Field
+from pydantic import BaseSettings, Field, validator
 
 from argilla._constants import DEFAULT_MAX_KEYWORD_LENGTH
 
@@ -60,6 +60,24 @@ class ApiSettings(BaseSettings):
     __DATASETS_INDEX_NAME__ = "ar.datasets"
     __DATASETS_RECORDS_INDEX_NAME__ = "ar.dataset.{}"
 
+    base_url: str = Field(
+        default="/",
+        description="The default base url where server will be deployed",
+    )
+
+    @validator("base_url")
+    @classmethod
+    def normalize_base_url(cls, value: str):
+        if not value:
+            value = "/"
+        if not value.startswith("/"):
+            value = "/" + value
+
+        if not value.endswith("/"):
+            value += "/"
+
+        return value
+
     elasticsearch: str = "http://localhost:9200"
     elasticsearch_ssl_verify: bool = True
     elasticsearch_ca_path: Optional[str] = None
@@ -83,6 +101,11 @@ class ApiSettings(BaseSettings):
     es_records_index_shards: int = 1
     es_records_index_replicas: int = 0
 
+    vectors_fields_limit: int = Field(
+        default=5,
+        description="Max number of supported vectors per record",
+    )
+
     metadata_fields_limit: int = Field(
         default=50,
         gt=0,
